@@ -9,9 +9,18 @@ import statistics
 from botocore.exceptions import ClientError
 
 
-def days_diff(dt, first_update_dt):
-    d1 = datetime.strptime(dt, "%Y-%m-%d")
-    d2 = datetime.strptime(first_update_dt, "%Y-%m-%d")
+def days_diff(d1, d2):
+    """
+    Returns days between dates:  d2 - d1
+
+    :param d1:
+    :param d2:
+    :return:
+    """
+    print("days_diff: formatting dates")
+    d1 = datetime.strptime(d1, "%Y-%m-%d")
+    d2 = datetime.strptime(d2, "%Y-%m-%d")
+    print("days_diff: subtracting dates")
     return (d2 - d1).days
 
 
@@ -34,6 +43,7 @@ def accuracy_metrics(vals, expected_dt, truth_val):
     avg = None
     std = None
     final_acc = None
+    normalized_acc = None
 
     if len(accs) > 1:
         std = statistics.stdev(accs)
@@ -45,18 +55,31 @@ def accuracy_metrics(vals, expected_dt, truth_val):
         orig_acc = accs[0]
         final_acc = accs[0]
 
+    if truth_val != 0:
+        normalized_acc = (1 / len(accs)) * (sum([1 - (acc / truth_val) for acc in accs]))
+
     return {
         "OriginalAccuracy": orig_acc,
         "AverageAccuracy": avg,
         "AccuracySTDev": std,
-        "FinalAccuracy": final_acc
+        "FinalAccuracy": final_acc,
+        "NormalizedAccuracy": normalized_acc
     }
 
 
-def completeness_metric(vals, expected_dt, truth_val):
-    expected_vals = vals[expected_dt:]
-    not_na = expected_vals[expected_vals.notna()]
-    return {"Completeness": len(not_na) / len(not_na)}
+def completeness_metric(series,  truth_df, expected_df, *args, **kwargs):
+    """
+    Returns percent of an update that includes previous values
+
+    :param vals:
+    :param expected_dt:
+    :param truth_val:
+    :return:
+    """
+    # check that there is an update for all the expected less than or equal to this
+    expected_vals = expected_df[expected_df.iloc[:, 0] < series.name].index
+    num_missing = len([each for each in expected_vals if each not in series.index])
+    return {"Completeness": 1 - (num_missing / len(expected_vals))}
 
 
 def time_till_metrics(vals, expected_dt, truth_val):
@@ -77,41 +100,34 @@ def time_till_metrics(vals, expected_dt, truth_val):
     # ------Time Till Accurate------- #
     # If final value was not correct
     if truth_val != vals_not_na[-1]:
-        return np.Inf
+        time_till_accurate = np.Inf  #  should this be something like more days than have elapsed?
 
-    # Else the final value was correct, and we need to find the last date for which it changed
+    else:
+        # Else the final value was correct, and we need to find the last date for which it changed
 
-    # changes between updates; [NaN, val1-val0, ...]
-    diffs = [each for each in vals_not_na.diff()]
-
-    # Find the dates at which the value changed
-    change_dates = [ind for ind, dff in zip(vals_not_na.index, diffs) if dff != 0]
+        # changes between updates; [NaN, val1-val0, ...]
+        diffs = [each for each in vals_not_na.diff()]
 
+        # Find the dates at which the value changed
+        change_dates = [ind for ind, dff in zip(vals_not_na.index, diffs) if dff != 0]
+        time_till_accurate = days_diff(expected_dt, change_dates[-1])
     return {
         "TimeTillReported": days_diff(expected_dt, first_reported),
-        "TimeTillAccurate": days_diff(expected_dt, change_dates[-1])
+        "TimeTillAccurate": time_till_accurate
     }
 
 
-# def dq_dimensions_observations(dt, row, final_val):
-#     not_na = row[row.notna()]
-#     if len(not_na) == 0:
-#         return {}
-#
-#     first_update_dt = not_na.index[0]
-#     first_val = not_na[0]
-#     last_update_dt = not_na.index[-1]
-#     return ({
-#         'timeTillVal': timeTillVal(dt, first_update_dt),
-#         'timeTillCorrect': timeTillCorrect(dt, row, final_val),
-#         'accuracyOriginal': accuracyOriginal(first_val, final_val),
-#         'accuracyAverage': accuracyAverage(row, final_val),
-#         'accuracyStdev': accuracyStdev(row, final_val),
-#         'completeness': completeness(row)
-#     })
+def validity_nondecreasing_smoothness(series,  truth_df, expected_df, *args, **kwargs):
+    """
+    Assumes series is of increasing dates by nondecreasing observations
 
-
-def validity_nondecreasing_smoothness(series, *args, **kwargs):
+    :param series:
+    :param truth_df:
+    :param expected_df:
+    :param args:
+    :param kwargs:
+    :return:
+    """
     orig_diffs = series.diff()
     rises = [abs(diff) for diff in orig_diffs if diff > 0]
     inds = [ind for ind, diff in enumerate(orig_diffs) if diff > 0]
@@ -121,13 +137,26 @@ def validity_nondecreasing_smoothness(series, *args, **kwargs):
 
     runs = [inds[0]]
     runs = runs + [abs(ind2 - ind1) for ind1, ind2 in zip(inds[:-1], inds[1:])]
-    return sum([rise * run / 2 for rise, run in zip(rises, runs)])
+
+    # last observation updated (max dt not na)
+    obsvs = series[series.notna()]
+    max_date = obsvs.index[-1]
+    max_val = obsvs.iloc[-1][0]
+    num_expected_dates = expected_df[expected_df.iloc[:, 0] < max_date].shape[0]
+    total_area = max_val * num_expected_dates
+    covered_area = sum([rise * run for rise, run in zip(rises, runs)])
+    return 1 - (covered_area / total_area)
 
 
 def validity_nondecreasing(series, *args, **kwargs):
     """Checks if values are nondecreasing.
     Return number of times a decrease occurs"""
-    return len(series[series.notna().diff() < 0])
+    num_decreasing_updates = len([each for each in series[series.notna()].diff() if each < 0])
+    num_updates = len(series[series.notna()])
+    if num_updates > 1:
+        return 1 - (num_decreasing_updates / (num_updates - 1))
+    else:
+        return 1
 
 
 # ----------------------------------------------#
@@ -262,7 +291,9 @@ def df_from_s3_csv(bucket, key):
     """
     s3_client = boto3.client('s3')
     logging.info(f'Attempting to read S3 object {key}')
+    print(f'Attempting to read S3 object {key} in {bucket}')
     obj = s3_client.get_object(Bucket=bucket, Key=key)
+    print(f'Successfully read S3 object {key} in {bucket}')
     logging.info(f'Successfully read S3 object {key}')
     df = pd.read_csv(obj['Body'], sep=',', index_col=0)
     return df
@@ -277,11 +308,14 @@ def df_to_s3_csv(bucket, key, df):
     :param df: pandas dataframe
     :return: True
     """
+    print("Connecting to S3")
     s3_client = boto3.client('s3')
 
+    print("Writing BytesIO")
     with BytesIO() as csv_buffer:
-        df.to_csv(csv_buffer)
 
+        df.to_csv(csv_buffer)
+        print(f'Attempting to put object to bucket {bucket} at key {key}')
         response = s3_client.put_object(Bucket=bucket, Key=key, Body=csv_buffer.getvalue())
 
         status = response.get("ResponseMetadata", {}).get("HTTPStatusCode")
@@ -411,6 +445,7 @@ def summary_node(
         context,
         to_sqs_url,
         region_name,
+        from_bucket,
         to_bucket,
         to_key
 ):
@@ -449,7 +484,7 @@ def summary_node(
     if 'expected_file' in body:
         expected_file = body['expected_file']
 
-    from_bucket = body['bucket']
+    # from_bucket = body['bucket']  # unless completed edits the event, this is prepared-data
 
     def make_key(file):
         if path == "":
